# coding: utf-8

"""
    Habitat repository and authorization API

    Habitat API  # noqa: E501

    OpenAPI spec version: V1.1.1
    Contact: zives@seas.upenn.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from pennprov.models.id_model import IDModel  # noqa: F401,E501
from pennprov.models.prov_specifier_model import ProvSpecifierModel  # noqa: F401,E501


class ProvLocationModel(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'stream': 'IDModel',
        'field': 'str',
        'position': 'list[int]'
    }

    attribute_map = {
        'stream': 'stream',
        'field': 'field',
        'position': 'position'
    }

    def __init__(self, stream=None, field=None, position=None):  # noqa: E501
        """ProvLocationModel - a model defined in Swagger"""  # noqa: E501

        self._stream = None
        self._field = None
        self._position = None
        self.discriminator = None

        if stream is not None:
            self.stream = stream
        self.field = field
        self.position = position

    @property
    def stream(self):
        """Gets the stream of this ProvLocationModel.  # noqa: E501


        :return: The stream of this ProvLocationModel.  # noqa: E501
        :rtype: IDModel
        """
        return self._stream

    @stream.setter
    def stream(self, stream):
        """Sets the stream of this ProvLocationModel.


        :param stream: The stream of this ProvLocationModel.  # noqa: E501
        :type: IDModel
        """

        self._stream = stream

    @property
    def field(self):
        """Gets the field of this ProvLocationModel.  # noqa: E501


        :return: The field of this ProvLocationModel.  # noqa: E501
        :rtype: str
        """
        return self._field

    @field.setter
    def field(self, field):
        """Sets the field of this ProvLocationModel.


        :param field: The field of this ProvLocationModel.  # noqa: E501
        :type: str
        """
        if field is None:
            raise ValueError("Invalid value for `field`, must not be `None`")  # noqa: E501

        self._field = field

    @property
    def position(self):
        """Gets the position of this ProvLocationModel.  # noqa: E501


        :return: The position of this ProvLocationModel.  # noqa: E501
        :rtype: list[int]
        """
        return self._position

    @position.setter
    def position(self, position):
        """Sets the position of this ProvLocationModel.


        :param position: The position of this ProvLocationModel.  # noqa: E501
        :type: list[int]
        """
        if position is None:
            raise ValueError("Invalid value for `position`, must not be `None`")  # noqa: E501

        self._position = position

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ProvLocationModel, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProvLocationModel):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Habitat repository and authorization API

    Habitat API  # noqa: E501

    OpenAPI spec version: V1.1.1
    Contact: zives@seas.upenn.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from pennprov.models.attribute import Attribute  # noqa: F401,E501
from pennprov.models.prov_specifier_model import ProvSpecifierModel  # noqa: F401,E501


class NodeModel(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'attributes': 'list[Attribute]',
        'type': 'str',
        'location': 'ProvSpecifierModel',
        'start_time': 'datetime',
        'end_time': 'datetime'
    }

    attribute_map = {
        'attributes': 'attributes',
        'type': 'type',
        'location': 'location',
        'start_time': 'startTime',
        'end_time': 'endTime'
    }

    def __init__(self, attributes=None, type=None, location=None, start_time=None, end_time=None):  # noqa: E501
        """NodeModel - a model defined in Swagger"""  # noqa: E501

        self._attributes = None
        self._type = None
        self._location = None
        self._start_time = None
        self._end_time = None
        self.discriminator = None

        if attributes is not None:
            self.attributes = attributes
        self.type = type
        if location is not None:
            self.location = location
        if start_time is not None:
            self.start_time = start_time
        if end_time is not None:
            self.end_time = end_time

    @property
    def attributes(self):
        """Gets the attributes of this NodeModel.  # noqa: E501


        :return: The attributes of this NodeModel.  # noqa: E501
        :rtype: list[Attribute]
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """Sets the attributes of this NodeModel.


        :param attributes: The attributes of this NodeModel.  # noqa: E501
        :type: list[Attribute]
        """

        self._attributes = attributes

    @property
    def type(self):
        """Gets the type of this NodeModel.  # noqa: E501


        :return: The type of this NodeModel.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this NodeModel.


        :param type: The type of this NodeModel.  # noqa: E501
        :type: str
        """
        if type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        allowed_values = ["ENTITY", "BUNDLE", "COLLECTION", "EMPTY_COLLECTION", "PLAN", "ACTIVITY", "AGENT", "ORGANIZATION", "PERSON", "SOFTWARE_AGENT"]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    @property
    def location(self):
        """Gets the location of this NodeModel.  # noqa: E501


        :return: The location of this NodeModel.  # noqa: E501
        :rtype: ProvSpecifierModel
        """
        return self._location

    @location.setter
    def location(self, location):
        """Sets the location of this NodeModel.


        :param location: The location of this NodeModel.  # noqa: E501
        :type: ProvSpecifierModel
        """

        self._location = location

    @property
    def start_time(self):
        """Gets the start_time of this NodeModel.  # noqa: E501

        it is an error to include start time unless type is ACTIVITY  # noqa: E501

        :return: The start_time of this NodeModel.  # noqa: E501
        :rtype: datetime
        """
        return self._start_time

    @start_time.setter
    def start_time(self, start_time):
        """Sets the start_time of this NodeModel.

        it is an error to include start time unless type is ACTIVITY  # noqa: E501

        :param start_time: The start_time of this NodeModel.  # noqa: E501
        :type: datetime
        """

        self._start_time = start_time

    @property
    def end_time(self):
        """Gets the end_time of this NodeModel.  # noqa: E501

        it is an error to include end time unless type is ACTIVITY  # noqa: E501

        :return: The end_time of this NodeModel.  # noqa: E501
        :rtype: datetime
        """
        return self._end_time

    @end_time.setter
    def end_time(self, end_time):
        """Sets the end_time of this NodeModel.

        it is an error to include end time unless type is ACTIVITY  # noqa: E501

        :param end_time: The end_time of this NodeModel.  # noqa: E501
        :type: datetime
        """

        self._end_time = end_time

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(NodeModel, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NodeModel):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Habitat repository and authorization API

    Habitat API  # noqa: E501

    OpenAPI spec version: V1.1.1
    Contact: zives@seas.upenn.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from pennprov.models.link_instance import LinkInstance  # noqa: F401,E501
from pennprov.models.rank_instance import RankInstance  # noqa: F401,E501


class SubgraphInstance(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'ranks': 'list[RankInstance]',
        'links': 'list[LinkInstance]',
        'max_timestamp': 'int'
    }

    attribute_map = {
        'ranks': 'ranks',
        'links': 'links',
        'max_timestamp': 'maxTimestamp'
    }

    def __init__(self, ranks=None, links=None, max_timestamp=None):  # noqa: E501
        """SubgraphInstance - a model defined in Swagger"""  # noqa: E501

        self._ranks = None
        self._links = None
        self._max_timestamp = None
        self.discriminator = None

        if ranks is not None:
            self.ranks = ranks
        if links is not None:
            self.links = links
        if max_timestamp is not None:
            self.max_timestamp = max_timestamp

    @property
    def ranks(self):
        """Gets the ranks of this SubgraphInstance.  # noqa: E501


        :return: The ranks of this SubgraphInstance.  # noqa: E501
        :rtype: list[RankInstance]
        """
        return self._ranks

    @ranks.setter
    def ranks(self, ranks):
        """Sets the ranks of this SubgraphInstance.


        :param ranks: The ranks of this SubgraphInstance.  # noqa: E501
        :type: list[RankInstance]
        """

        self._ranks = ranks

    @property
    def links(self):
        """Gets the links of this SubgraphInstance.  # noqa: E501


        :return: The links of this SubgraphInstance.  # noqa: E501
        :rtype: list[LinkInstance]
        """
        return self._links

    @links.setter
    def links(self, links):
        """Sets the links of this SubgraphInstance.


        :param links: The links of this SubgraphInstance.  # noqa: E501
        :type: list[LinkInstance]
        """

        self._links = links

    @property
    def max_timestamp(self):
        """Gets the max_timestamp of this SubgraphInstance.  # noqa: E501


        :return: The max_timestamp of this SubgraphInstance.  # noqa: E501
        :rtype: int
        """
        return self._max_timestamp

    @max_timestamp.setter
    def max_timestamp(self, max_timestamp):
        """Sets the max_timestamp of this SubgraphInstance.


        :param max_timestamp: The max_timestamp of this SubgraphInstance.  # noqa: E501
        :type: int
        """

        self._max_timestamp = max_timestamp

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(SubgraphInstance, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SubgraphInstance):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Habitat repository and authorization API

    Habitat API  # noqa: E501

    OpenAPI spec version: V1.1.1
    Contact: zives@seas.upenn.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class FieldModel(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'name': 'str',
        'value': 'object',
        'type': 'str'
    }

    attribute_map = {
        'name': 'name',
        'value': 'value',
        'type': 'type'
    }

    discriminator_value_class_map = {
        'ProvTokenFieldModel': 'ProvTokenFieldModel',
        'BooleanFieldModel': 'BooleanFieldModel',
        'QualifiedNameFieldModel': 'QualifiedNameFieldModel',
        'ProvSpecifierFieldModel': 'ProvSpecifierFieldModel',
        'StringFieldModel': 'StringFieldModel',
        'DoubleFieldModel': 'DoubleFieldModel',
        'MultiFieldModel': 'MultiFieldModel',
        'IntegerFieldModel': 'IntegerFieldModel',
        'LongFieldModel': 'LongFieldModel'
    }

    def __init__(self, name=None, value=None, type=None):  # noqa: E501
        """FieldModel - a model defined in Swagger"""  # noqa: E501

        self._name = None
        self._value = None
        self._type = None
        self.discriminator = 'type'

        self.name = name
        self.value = value
        self.type = type

    @property
    def name(self):
        """Gets the name of this FieldModel.  # noqa: E501


        :return: The name of this FieldModel.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this FieldModel.


        :param name: The name of this FieldModel.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def value(self):
        """Gets the value of this FieldModel.  # noqa: E501


        :return: The value of this FieldModel.  # noqa: E501
        :rtype: object
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this FieldModel.


        :param value: The value of this FieldModel.  # noqa: E501
        :type: object
        """
        if value is None:
            raise ValueError("Invalid value for `value`, must not be `None`")  # noqa: E501

        self._value = value

    @property
    def type(self):
        """Gets the type of this FieldModel.  # noqa: E501


        :return: The type of this FieldModel.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this FieldModel.


        :param type: The type of this FieldModel.  # noqa: E501
        :type: str
        """
        if type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        allowed_values = ["BOOLEAN", "DOUBLE", "INTEGER", "STRING", "PROV_TOKEN", "PROV_SPECIFIER", "LONG", "QUALIFIED_NAME", "MULTI"]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    def get_real_child_model(self, data):
        """Returns the real base class specified by the discriminator"""
        discriminator_value = data[self.discriminator].lower()
        return self.discriminator_value_class_map.get(discriminator_value)

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(FieldModel, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, FieldModel):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Habitat repository and authorization API

    Habitat API  # noqa: E501

    OpenAPI spec version: V1.1.1
    Contact: zives@seas.upenn.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class UserInfo(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'username': 'str',
        'email': 'str',
        'password': 'str',
        'firstname': 'str',
        'lastname': 'str',
        'street1': 'str',
        'street2': 'str',
        'city': 'str',
        'state': 'str',
        'zip': 'str',
        'country': 'str',
        'phone': 'str',
        'title': 'str',
        'organization': 'str'
    }

    attribute_map = {
        'username': 'username',
        'email': 'email',
        'password': 'password',
        'firstname': 'firstname',
        'lastname': 'lastname',
        'street1': 'street1',
        'street2': 'street2',
        'city': 'city',
        'state': 'state',
        'zip': 'zip',
        'country': 'country',
        'phone': 'phone',
        'title': 'title',
        'organization': 'organization'
    }

    def __init__(self, username=None, email=None, password=None, firstname=None, lastname=None, street1=None, street2=None, city=None, state=None, zip=None, country=None, phone=None, title=None, organization=None):  # noqa: E501
        """UserInfo - a model defined in Swagger"""  # noqa: E501

        self._username = None
        self._email = None
        self._password = None
        self._firstname = None
        self._lastname = None
        self._street1 = None
        self._street2 = None
        self._city = None
        self._state = None
        self._zip = None
        self._country = None
        self._phone = None
        self._title = None
        self._organization = None
        self.discriminator = None

        if username is not None:
            self.username = username
        if email is not None:
            self.email = email
        if password is not None:
            self.password = password
        if firstname is not None:
            self.firstname = firstname
        if lastname is not None:
            self.lastname = lastname
        if street1 is not None:
            self.street1 = street1
        if street2 is not None:
            self.street2 = street2
        if city is not None:
            self.city = city
        if state is not None:
            self.state = state
        if zip is not None:
            self.zip = zip
        if country is not None:
            self.country = country
        if phone is not None:
            self.phone = phone
        if title is not None:
            self.title = title
        if organization is not None:
            self.organization = organization

    @property
    def username(self):
        """Gets the username of this UserInfo.  # noqa: E501


        :return: The username of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._username

    @username.setter
    def username(self, username):
        """Sets the username of this UserInfo.


        :param username: The username of this UserInfo.  # noqa: E501
        :type: str
        """

        self._username = username

    @property
    def email(self):
        """Gets the email of this UserInfo.  # noqa: E501


        :return: The email of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """Sets the email of this UserInfo.


        :param email: The email of this UserInfo.  # noqa: E501
        :type: str
        """

        self._email = email

    @property
    def password(self):
        """Gets the password of this UserInfo.  # noqa: E501


        :return: The password of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._password

    @password.setter
    def password(self, password):
        """Sets the password of this UserInfo.


        :param password: The password of this UserInfo.  # noqa: E501
        :type: str
        """

        self._password = password

    @property
    def firstname(self):
        """Gets the firstname of this UserInfo.  # noqa: E501


        :return: The firstname of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._firstname

    @firstname.setter
    def firstname(self, firstname):
        """Sets the firstname of this UserInfo.


        :param firstname: The firstname of this UserInfo.  # noqa: E501
        :type: str
        """

        self._firstname = firstname

    @property
    def lastname(self):
        """Gets the lastname of this UserInfo.  # noqa: E501


        :return: The lastname of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._lastname

    @lastname.setter
    def lastname(self, lastname):
        """Sets the lastname of this UserInfo.


        :param lastname: The lastname of this UserInfo.  # noqa: E501
        :type: str
        """

        self._lastname = lastname

    @property
    def street1(self):
        """Gets the street1 of this UserInfo.  # noqa: E501


        :return: The street1 of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._street1

    @street1.setter
    def street1(self, street1):
        """Sets the street1 of this UserInfo.


        :param street1: The street1 of this UserInfo.  # noqa: E501
        :type: str
        """

        self._street1 = street1

    @property
    def street2(self):
        """Gets the street2 of this UserInfo.  # noqa: E501


        :return: The street2 of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._street2

    @street2.setter
    def street2(self, street2):
        """Sets the street2 of this UserInfo.


        :param street2: The street2 of this UserInfo.  # noqa: E501
        :type: str
        """

        self._street2 = street2

    @property
    def city(self):
        """Gets the city of this UserInfo.  # noqa: E501


        :return: The city of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._city

    @city.setter
    def city(self, city):
        """Sets the city of this UserInfo.


        :param city: The city of this UserInfo.  # noqa: E501
        :type: str
        """

        self._city = city

    @property
    def state(self):
        """Gets the state of this UserInfo.  # noqa: E501


        :return: The state of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this UserInfo.


        :param state: The state of this UserInfo.  # noqa: E501
        :type: str
        """

        self._state = state

    @property
    def zip(self):
        """Gets the zip of this UserInfo.  # noqa: E501


        :return: The zip of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._zip

    @zip.setter
    def zip(self, zip):
        """Sets the zip of this UserInfo.


        :param zip: The zip of this UserInfo.  # noqa: E501
        :type: str
        """

        self._zip = zip

    @property
    def country(self):
        """Gets the country of this UserInfo.  # noqa: E501


        :return: The country of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._country

    @country.setter
    def country(self, country):
        """Sets the country of this UserInfo.


        :param country: The country of this UserInfo.  # noqa: E501
        :type: str
        """

        self._country = country

    @property
    def phone(self):
        """Gets the phone of this UserInfo.  # noqa: E501


        :return: The phone of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._phone

    @phone.setter
    def phone(self, phone):
        """Sets the phone of this UserInfo.


        :param phone: The phone of this UserInfo.  # noqa: E501
        :type: str
        """

        self._phone = phone

    @property
    def title(self):
        """Gets the title of this UserInfo.  # noqa: E501


        :return: The title of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this UserInfo.


        :param title: The title of this UserInfo.  # noqa: E501
        :type: str
        """

        self._title = title

    @property
    def organization(self):
        """Gets the organization of this UserInfo.  # noqa: E501


        :return: The organization of this UserInfo.  # noqa: E501
        :rtype: str
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this UserInfo.


        :param organization: The organization of this UserInfo.  # noqa: E501
        :type: str
        """

        self._organization = organization

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(UserInfo, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UserInfo):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Habitat repository and authorization API

    Habitat API  # noqa: E501

    OpenAPI spec version: V1.1.1
    Contact: zives@seas.upenn.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from pennprov.api_client import ApiClient


class AuthenticationApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def add_credential(self, service, username, credentials, **kwargs):  # noqa: E501
        """Adds a service credential for the user  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_credential(service, username, credentials, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str service: (required)
        :param str username: (required)
        :param UserCredentials credentials: (required)
        :return: UserInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_credential_with_http_info(service, username, credentials, **kwargs)  # noqa: E501
        else:
            (data) = self.add_credential_with_http_info(service, username, credentials, **kwargs)  # noqa: E501
            return data

    def add_credential_with_http_info(self, service, username, credentials, **kwargs):  # noqa: E501
        """Adds a service credential for the user  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_credential_with_http_info(service, username, credentials, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str service: (required)
        :param str username: (required)
        :param UserCredentials credentials: (required)
        :return: UserInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['service', 'username', 'credentials']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_credential" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'service' is set
        if ('service' not in params or
                params['service'] is None):
            raise ValueError("Missing the required parameter `service` when calling `add_credential`")  # noqa: E501
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `add_credential`")  # noqa: E501
        # verify the required parameter 'credentials' is set
        if ('credentials' not in params or
                params['credentials'] is None):
            raise ValueError("Missing the required parameter `credentials` when calling `add_credential`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'service' in params:
            path_params['service'] = params['service']  # noqa: E501
        if 'username' in params:
            path_params['username'] = params['username']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'credentials' in params:
            body_params = params['credentials']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/services/{service}/user/{username}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='UserInfo',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def add_group(self, organization, group, details, **kwargs):  # noqa: E501
        """Adds a permissions group to an organization  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_group(organization, group, details, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int organization: (required)
        :param str group: (required)
        :param GroupDetails details: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_group_with_http_info(organization, group, details, **kwargs)  # noqa: E501
        else:
            (data) = self.add_group_with_http_info(organization, group, details, **kwargs)  # noqa: E501
            return data

    def add_group_with_http_info(self, organization, group, details, **kwargs):  # noqa: E501
        """Adds a permissions group to an organization  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_group_with_http_info(organization, group, details, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int organization: (required)
        :param str group: (required)
        :param GroupDetails details: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['organization', 'group', 'details']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_group" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'organization' is set
        if ('organization' not in params or
                params['organization'] is None):
            raise ValueError("Missing the required parameter `organization` when calling `add_group`")  # noqa: E501
        # verify the required parameter 'group' is set
        if ('group' not in params or
                params['group'] is None):
            raise ValueError("Missing the required parameter `group` when calling `add_group`")  # noqa: E501
        # verify the required parameter 'details' is set
        if ('details' not in params or
                params['details'] is None):
            raise ValueError("Missing the required parameter `details` when calling `add_group`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'organization' in params:
            path_params['organization'] = params['organization']  # noqa: E501
        if 'group' in params:
            path_params['group'] = params['group']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'details' in params:
            body_params = params['details']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/organizations/{organization}/group/{group}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='int',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def add_organization(self, orgname, details, **kwargs):  # noqa: E501
        """Adds an organization  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_organization(orgname, details, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str orgname: (required)
        :param OrgDetails details: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_organization_with_http_info(orgname, details, **kwargs)  # noqa: E501
        else:
            (data) = self.add_organization_with_http_info(orgname, details, **kwargs)  # noqa: E501
            return data

    def add_organization_with_http_info(self, orgname, details, **kwargs):  # noqa: E501
        """Adds an organization  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_organization_with_http_info(orgname, details, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str orgname: (required)
        :param OrgDetails details: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['orgname', 'details']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_organization" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'orgname' is set
        if ('orgname' not in params or
                params['orgname'] is None):
            raise ValueError("Missing the required parameter `orgname` when calling `add_organization`")  # noqa: E501
        # verify the required parameter 'details' is set
        if ('details' not in params or
                params['details'] is None):
            raise ValueError("Missing the required parameter `details` when calling `add_organization`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'orgname' in params:
            path_params['orgname'] = params['orgname']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'details' in params:
            body_params = params['details']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/organizations/{orgname}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='int',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def add_subgroup_to_group(self, groupname, subgroup, **kwargs):  # noqa: E501
        """Adds a subgroup to a group  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_subgroup_to_group(groupname, subgroup, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str groupname: (required)
        :param str subgroup: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_subgroup_to_group_with_http_info(groupname, subgroup, **kwargs)  # noqa: E501
        else:
            (data) = self.add_subgroup_to_group_with_http_info(groupname, subgroup, **kwargs)  # noqa: E501
            return data

    def add_subgroup_to_group_with_http_info(self, groupname, subgroup, **kwargs):  # noqa: E501
        """Adds a subgroup to a group  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_subgroup_to_group_with_http_info(groupname, subgroup, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str groupname: (required)
        :param str subgroup: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['groupname', 'subgroup']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_subgroup_to_group" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'groupname' is set
        if ('groupname' not in params or
                params['groupname'] is None):
            raise ValueError("Missing the required parameter `groupname` when calling `add_subgroup_to_group`")  # noqa: E501
        # verify the required parameter 'subgroup' is set
        if ('subgroup' not in params or
                params['subgroup'] is None):
            raise ValueError("Missing the required parameter `subgroup` when calling `add_subgroup_to_group`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'groupname' in params:
            path_params['groupname'] = params['groupname']  # noqa: E501
        if 'subgroup' in params:
            path_params['subgroup'] = params['subgroup']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/groups/{groupname}/group/{subgroup}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='int',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def add_user_to_group(self, groupname, username, **kwargs):  # noqa: E501
        """Adds a user to a group  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_user_to_group(groupname, username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str groupname: (required)
        :param str username: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_user_to_group_with_http_info(groupname, username, **kwargs)  # noqa: E501
        else:
            (data) = self.add_user_to_group_with_http_info(groupname, username, **kwargs)  # noqa: E501
            return data

    def add_user_to_group_with_http_info(self, groupname, username, **kwargs):  # noqa: E501
        """Adds a user to a group  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_user_to_group_with_http_info(groupname, username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str groupname: (required)
        :param str username: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['groupname', 'username']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_user_to_group" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'groupname' is set
        if ('groupname' not in params or
                params['groupname'] is None):
            raise ValueError("Missing the required parameter `groupname` when calling `add_user_to_group`")  # noqa: E501
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `add_user_to_group`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'groupname' in params:
            path_params['groupname'] = params['groupname']  # noqa: E501
        if 'username' in params:
            path_params['username'] = params['username']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/groups/{groupname}/user/{username}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='int',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def add_user_to_organization(self, org_id, username, **kwargs):  # noqa: E501
        """Adds a user to an organization  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_user_to_organization(org_id, username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int org_id: (required)
        :param str username: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_user_to_organization_with_http_info(org_id, username, **kwargs)  # noqa: E501
        else:
            (data) = self.add_user_to_organization_with_http_info(org_id, username, **kwargs)  # noqa: E501
            return data

    def add_user_to_organization_with_http_info(self, org_id, username, **kwargs):  # noqa: E501
        """Adds a user to an organization  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_user_to_organization_with_http_info(org_id, username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int org_id: (required)
        :param str username: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['org_id', 'username']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_user_to_organization" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'org_id' is set
        if ('org_id' not in params or
                params['org_id'] is None):
            raise ValueError("Missing the required parameter `org_id` when calling `add_user_to_organization`")  # noqa: E501
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `add_user_to_organization`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'org_id' in params:
            path_params['orgId'] = params['org_id']  # noqa: E501
        if 'username' in params:
            path_params['username'] = params['username']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/organizations/{orgId}/user/{username}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='int',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_new_user(self, username, userfields, **kwargs):  # noqa: E501
        """Creates a new user  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_new_user(username, userfields, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :param UserInfo userfields: (required)
        :return: UserInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_new_user_with_http_info(username, userfields, **kwargs)  # noqa: E501
        else:
            (data) = self.create_new_user_with_http_info(username, userfields, **kwargs)  # noqa: E501
            return data

    def create_new_user_with_http_info(self, username, userfields, **kwargs):  # noqa: E501
        """Creates a new user  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_new_user_with_http_info(username, userfields, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :param UserInfo userfields: (required)
        :return: UserInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'userfields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_new_user" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `create_new_user`")  # noqa: E501
        # verify the required parameter 'userfields' is set
        if ('userfields' not in params or
                params['userfields'] is None):
            raise ValueError("Missing the required parameter `userfields` when calling `create_new_user`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'username' in params:
            path_params['username'] = params['username']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'userfields' in params:
            body_params = params['userfields']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/auth/credentials/{username}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='UserInfo',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_group_from_id(self, group_id, **kwargs):  # noqa: E501
        """Gets a group name from its integer ID  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_group_from_id(group_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int group_id: (required)
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_group_from_id_with_http_info(group_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_group_from_id_with_http_info(group_id, **kwargs)  # noqa: E501
            return data

    def get_group_from_id_with_http_info(self, group_id, **kwargs):  # noqa: E501
        """Gets a group name from its integer ID  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_group_from_id_with_http_info(group_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int group_id: (required)
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['group_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_group_from_id" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'group_id' is set
        if ('group_id' not in params or
                params['group_id'] is None):
            raise ValueError("Missing the required parameter `group_id` when calling `get_group_from_id`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'group_id' in params:
            path_params['groupId'] = params['group_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/groups/id/{groupId}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='str',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_group_id(self, group_name, **kwargs):  # noqa: E501
        """Gets a group's ID from its name  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_group_id(group_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str group_name: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_group_id_with_http_info(group_name, **kwargs)  # noqa: E501
        else:
            (data) = self.get_group_id_with_http_info(group_name, **kwargs)  # noqa: E501
            return data

    def get_group_id_with_http_info(self, group_name, **kwargs):  # noqa: E501
        """Gets a group's ID from its name  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_group_id_with_http_info(group_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str group_name: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['group_name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_group_id" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'group_name' is set
        if ('group_name' not in params or
                params['group_name'] is None):
            raise ValueError("Missing the required parameter `group_name` when calling `get_group_id`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'group_name' in params:
            path_params['groupName'] = params['group_name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/groups/name/{groupName}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='int',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_group_ids_for_user(self, username, **kwargs):  # noqa: E501
        """Gets the IDs of groups in which a user is directly a member  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_group_ids_for_user(username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :return: list[object]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_group_ids_for_user_with_http_info(username, **kwargs)  # noqa: E501
        else:
            (data) = self.get_group_ids_for_user_with_http_info(username, **kwargs)  # noqa: E501
            return data

    def get_group_ids_for_user_with_http_info(self, username, **kwargs):  # noqa: E501
        """Gets the IDs of groups in which a user is directly a member  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_group_ids_for_user_with_http_info(username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :return: list[object]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_group_ids_for_user" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `get_group_ids_for_user`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'username' in params:
            path_params['username'] = params['username']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/credentials/{username}/groups/id', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[object]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_groups_for_user(self, username, **kwargs):  # noqa: E501
        """Gets the groups in which a user is directly a member  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_groups_for_user(username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :return: list[object]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_groups_for_user_with_http_info(username, **kwargs)  # noqa: E501
        else:
            (data) = self.get_groups_for_user_with_http_info(username, **kwargs)  # noqa: E501
            return data

    def get_groups_for_user_with_http_info(self, username, **kwargs):  # noqa: E501
        """Gets the groups in which a user is directly a member  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_groups_for_user_with_http_info(username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :return: list[object]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_groups_for_user" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `get_groups_for_user`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'username' in params:
            path_params['username'] = params['username']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/credentials/{username}/groups', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[object]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_organization_from_id(self, org_id, **kwargs):  # noqa: E501
        """Gets an organization name from its integer ID  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_organization_from_id(org_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int org_id: (required)
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_organization_from_id_with_http_info(org_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_organization_from_id_with_http_info(org_id, **kwargs)  # noqa: E501
            return data

    def get_organization_from_id_with_http_info(self, org_id, **kwargs):  # noqa: E501
        """Gets an organization name from its integer ID  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_organization_from_id_with_http_info(org_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int org_id: (required)
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['org_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_organization_from_id" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'org_id' is set
        if ('org_id' not in params or
                params['org_id'] is None):
            raise ValueError("Missing the required parameter `org_id` when calling `get_organization_from_id`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'org_id' in params:
            path_params['orgId'] = params['org_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/organizations/id/{orgId}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='str',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_organization_id(self, org_name, **kwargs):  # noqa: E501
        """Gets an organization ID from its name  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_organization_id(org_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str org_name: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_organization_id_with_http_info(org_name, **kwargs)  # noqa: E501
        else:
            (data) = self.get_organization_id_with_http_info(org_name, **kwargs)  # noqa: E501
            return data

    def get_organization_id_with_http_info(self, org_name, **kwargs):  # noqa: E501
        """Gets an organization ID from its name  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_organization_id_with_http_info(org_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str org_name: (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['org_name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_organization_id" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'org_name' is set
        if ('org_name' not in params or
                params['org_name'] is None):
            raise ValueError("Missing the required parameter `org_name` when calling `get_organization_id`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'org_name' in params:
            path_params['orgName'] = params['org_name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/organizations/name/{orgName}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='int',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_parent_group_ids(self, group_id, **kwargs):  # noqa: E501
        """Gets the IDs of parent groups  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_parent_group_ids(group_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int group_id: (required)
        :return: list[object]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_parent_group_ids_with_http_info(group_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_parent_group_ids_with_http_info(group_id, **kwargs)  # noqa: E501
            return data

    def get_parent_group_ids_with_http_info(self, group_id, **kwargs):  # noqa: E501
        """Gets the IDs of parent groups  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_parent_group_ids_with_http_info(group_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int group_id: (required)
        :return: list[object]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['group_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_parent_group_ids" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'group_id' is set
        if ('group_id' not in params or
                params['group_id'] is None):
            raise ValueError("Missing the required parameter `group_id` when calling `get_parent_group_ids`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'group_id' in params:
            path_params['groupId'] = params['group_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/groups/id/{groupId}/parents/id', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[object]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_parent_groups(self, group_name, **kwargs):  # noqa: E501
        """Gets the groups in which a user is directly a member  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_parent_groups(group_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str group_name: (required)
        :return: list[object]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_parent_groups_with_http_info(group_name, **kwargs)  # noqa: E501
        else:
            (data) = self.get_parent_groups_with_http_info(group_name, **kwargs)  # noqa: E501
            return data

    def get_parent_groups_with_http_info(self, group_name, **kwargs):  # noqa: E501
        """Gets the groups in which a user is directly a member  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_parent_groups_with_http_info(group_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str group_name: (required)
        :return: list[object]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['group_name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_parent_groups" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'group_name' is set
        if ('group_name' not in params or
                params['group_name'] is None):
            raise ValueError("Missing the required parameter `group_name` when calling `get_parent_groups`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'group_name' in params:
            path_params['groupName'] = params['group_name']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/groups/name/{groupName}/parents/name', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[object]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_token_route(self, username, credentials, **kwargs):  # noqa: E501
        """Requests a new token  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_token_route(username, credentials, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :param UserCredentials credentials: (required)
        :return: WebToken
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_token_route_with_http_info(username, credentials, **kwargs)  # noqa: E501
        else:
            (data) = self.get_token_route_with_http_info(username, credentials, **kwargs)  # noqa: E501
            return data

    def get_token_route_with_http_info(self, username, credentials, **kwargs):  # noqa: E501
        """Requests a new token  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_token_route_with_http_info(username, credentials, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :param UserCredentials credentials: (required)
        :return: WebToken
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'credentials']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_token_route" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `get_token_route`")  # noqa: E501
        # verify the required parameter 'credentials' is set
        if ('credentials' not in params or
                params['credentials'] is None):
            raise ValueError("Missing the required parameter `credentials` when calling `get_token_route`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'username' in params:
            path_params['username'] = params['username']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'credentials' in params:
            body_params = params['credentials']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/auth/tokens/{username}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='WebToken',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_user_info(self, username, **kwargs):  # noqa: E501
        """Gets a user's info  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_user_info(username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :return: UserInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_user_info_with_http_info(username, **kwargs)  # noqa: E501
        else:
            (data) = self.get_user_info_with_http_info(username, **kwargs)  # noqa: E501
            return data

    def get_user_info_with_http_info(self, username, **kwargs):  # noqa: E501
        """Gets a user's info  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_user_info_with_http_info(username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :return: UserInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_user_info" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `get_user_info`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'username' in params:
            path_params['username'] = params['username']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/credentials/{username}/info', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='UserInfo',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def is_registered(self, service, username, **kwargs):  # noqa: E501
        """Returns whether a user credential is valid for a service  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.is_registered(service, username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str service: (required)
        :param str username: (required)
        :return: bool
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.is_registered_with_http_info(service, username, **kwargs)  # noqa: E501
        else:
            (data) = self.is_registered_with_http_info(service, username, **kwargs)  # noqa: E501
            return data

    def is_registered_with_http_info(self, service, username, **kwargs):  # noqa: E501
        """Returns whether a user credential is valid for a service  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.is_registered_with_http_info(service, username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str service: (required)
        :param str username: (required)
        :return: bool
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['service', 'username']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method is_registered" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'service' is set
        if ('service' not in params or
                params['service'] is None):
            raise ValueError("Missing the required parameter `service` when calling `is_registered`")  # noqa: E501
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `is_registered`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'service' in params:
            path_params['service'] = params['service']  # noqa: E501
        if 'username' in params:
            path_params['username'] = params['username']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/services/{service}/user/{username}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='bool',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def is_valid_credential(self, username, credentials, **kwargs):  # noqa: E501
        """Returns whether a local user credential is valid  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.is_valid_credential(username, credentials, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :param UserCredentials credentials: (required)
        :return: bool
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.is_valid_credential_with_http_info(username, credentials, **kwargs)  # noqa: E501
        else:
            (data) = self.is_valid_credential_with_http_info(username, credentials, **kwargs)  # noqa: E501
            return data

    def is_valid_credential_with_http_info(self, username, credentials, **kwargs):  # noqa: E501
        """Returns whether a local user credential is valid  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.is_valid_credential_with_http_info(username, credentials, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :param UserCredentials credentials: (required)
        :return: bool
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'credentials']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method is_valid_credential" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `is_valid_credential`")  # noqa: E501
        # verify the required parameter 'credentials' is set
        if ('credentials' not in params or
                params['credentials'] is None):
            raise ValueError("Missing the required parameter `credentials` when calling `is_valid_credential`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'username' in params:
            path_params['username'] = params['username']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'credentials' in params:
            body_params = params['credentials']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/services/local/user/{username}/credential', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='bool',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def update_user(self, username, userfields, **kwargs):  # noqa: E501
        """Updates user properties  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_user(username, userfields, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :param UserInfo userfields: (required)
        :return: UserInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.update_user_with_http_info(username, userfields, **kwargs)  # noqa: E501
        else:
            (data) = self.update_user_with_http_info(username, userfields, **kwargs)  # noqa: E501
            return data

    def update_user_with_http_info(self, username, userfields, **kwargs):  # noqa: E501
        """Updates user properties  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_user_with_http_info(username, userfields, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: (required)
        :param UserInfo userfields: (required)
        :return: UserInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'userfields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_user" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `update_user`")  # noqa: E501
        # verify the required parameter 'userfields' is set
        if ('userfields' not in params or
                params['userfields'] is None):
            raise ValueError("Missing the required parameter `userfields` when calling `update_user`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'username' in params:
            path_params['username'] = params['username']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'userfields' in params:
            body_params = params['userfields']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['jwt']  # noqa: E501

        return self.api_client.call_api(
            '/auth/credentials/{username}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='UserInfo',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8

"""
    Habitat repository and authorization API

    Habitat API  # noqa: E501

    OpenAPI spec version: V1.1.1
    Contact: zives@seas.upenn.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from pennprov.models.attribute import Attribute  # noqa: F401,E501
from pennprov.models.qualified_name import QualifiedName  # noqa: F401,E501


class RelationModel(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'attributes': 'list[Attribute]',
        'type': 'str',
        'subject_id': 'QualifiedName',
        'object_id': 'QualifiedName',
        'secondary_object_id': 'QualifiedName',
        'relation_id': 'QualifiedName',
        'time': 'datetime',
        'generation_id': 'QualifiedName',
        'usage_id': 'QualifiedName'
    }

    attribute_map = {
        'attributes': 'attributes',
        'type': 'type',
        'subject_id': 'subjectId',
        'object_id': 'objectId',
        'secondary_object_id': 'secondaryObjectId',
        'relation_id': 'relationId',
        'time': 'time',
        'generation_id': 'generationId',
        'usage_id': 'usageId'
    }

    def __init__(self, attributes=None, type=None, subject_id=None, object_id=None, secondary_object_id=None, relation_id=None, time=None, generation_id=None, usage_id=None):  # noqa: E501
        """RelationModel - a model defined in Swagger"""  # noqa: E501

        self._attributes = None
        self._type = None
        self._subject_id = None
        self._object_id = None
        self._secondary_object_id = None
        self._relation_id = None
        self._time = None
        self._generation_id = None
        self._usage_id = None
        self.discriminator = None

        if attributes is not None:
            self.attributes = attributes
        self.type = type
        self.subject_id = subject_id
        self.object_id = object_id
        if secondary_object_id is not None:
            self.secondary_object_id = secondary_object_id
        if relation_id is not None:
            self.relation_id = relation_id
        if time is not None:
            self.time = time
        if generation_id is not None:
            self.generation_id = generation_id
        if usage_id is not None:
            self.usage_id = usage_id

    @property
    def attributes(self):
        """Gets the attributes of this RelationModel.  # noqa: E501

        It is an error to include attributes in a MEMEBERSHIP, ALTERNATE, or SPECIALIZATION relation  # noqa: E501

        :return: The attributes of this RelationModel.  # noqa: E501
        :rtype: list[Attribute]
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """Sets the attributes of this RelationModel.

        It is an error to include attributes in a MEMEBERSHIP, ALTERNATE, or SPECIALIZATION relation  # noqa: E501

        :param attributes: The attributes of this RelationModel.  # noqa: E501
        :type: list[Attribute]
        """

        self._attributes = attributes

    @property
    def type(self):
        """Gets the type of this RelationModel.  # noqa: E501


        :return: The type of this RelationModel.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this RelationModel.


        :param type: The type of this RelationModel.  # noqa: E501
        :type: str
        """
        if type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        allowed_values = ["GENERATION", "USAGE", "COMMUNICATION", "START", "END", "INVALIDATION", "DERIVATION", "ATTRIBUTION", "ASSOCIATION", "DELEGATION", "INFLUENCE", "SPECIALIZATION", "ALTERNATE", "ANNOTATED", "MEMBERSHIP"]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    @property
    def subject_id(self):
        """Gets the subject_id of this RelationModel.  # noqa: E501


        :return: The subject_id of this RelationModel.  # noqa: E501
        :rtype: QualifiedName
        """
        return self._subject_id

    @subject_id.setter
    def subject_id(self, subject_id):
        """Sets the subject_id of this RelationModel.


        :param subject_id: The subject_id of this RelationModel.  # noqa: E501
        :type: QualifiedName
        """
        if subject_id is None:
            raise ValueError("Invalid value for `subject_id`, must not be `None`")  # noqa: E501

        self._subject_id = subject_id

    @property
    def object_id(self):
        """Gets the object_id of this RelationModel.  # noqa: E501


        :return: The object_id of this RelationModel.  # noqa: E501
        :rtype: QualifiedName
        """
        return self._object_id

    @object_id.setter
    def object_id(self, object_id):
        """Sets the object_id of this RelationModel.


        :param object_id: The object_id of this RelationModel.  # noqa: E501
        :type: QualifiedName
        """
        if object_id is None:
            raise ValueError("Invalid value for `object_id`, must not be `None`")  # noqa: E501

        self._object_id = object_id

    @property
    def secondary_object_id(self):
        """Gets the secondary_object_id of this RelationModel.  # noqa: E501

        It is an error to include a secondary object in a relation other than ASSOCIATION, DERIVATION, START, END, or DELEGATION  # noqa: E501

        :return: The secondary_object_id of this RelationModel.  # noqa: E501
        :rtype: QualifiedName
        """
        return self._secondary_object_id

    @secondary_object_id.setter
    def secondary_object_id(self, secondary_object_id):
        """Sets the secondary_object_id of this RelationModel.

        It is an error to include a secondary object in a relation other than ASSOCIATION, DERIVATION, START, END, or DELEGATION  # noqa: E501

        :param secondary_object_id: The secondary_object_id of this RelationModel.  # noqa: E501
        :type: QualifiedName
        """

        self._secondary_object_id = secondary_object_id

    @property
    def relation_id(self):
        """Gets the relation_id of this RelationModel.  # noqa: E501

        It is an error to include attributes in a MEMEBERSHIP, ALTERNATE, or SPECIALIZATION relation  # noqa: E501

        :return: The relation_id of this RelationModel.  # noqa: E501
        :rtype: QualifiedName
        """
        return self._relation_id

    @relation_id.setter
    def relation_id(self, relation_id):
        """Sets the relation_id of this RelationModel.

        It is an error to include attributes in a MEMEBERSHIP, ALTERNATE, or SPECIALIZATION relation  # noqa: E501

        :param relation_id: The relation_id of this RelationModel.  # noqa: E501
        :type: QualifiedName
        """

        self._relation_id = relation_id

    @property
    def time(self):
        """Gets the time of this RelationModel.  # noqa: E501

        It is an error to include time in a relation other than GENERATION, USAGE, START, END, or INVALIDATION  # noqa: E501

        :return: The time of this RelationModel.  # noqa: E501
        :rtype: datetime
        """
        return self._time

    @time.setter
    def time(self, time):
        """Sets the time of this RelationModel.

        It is an error to include time in a relation other than GENERATION, USAGE, START, END, or INVALIDATION  # noqa: E501

        :param time: The time of this RelationModel.  # noqa: E501
        :type: datetime
        """

        self._time = time

    @property
    def generation_id(self):
        """Gets the generation_id of this RelationModel.  # noqa: E501

        It is an error to include a generation id in a relation other than DERIVATION  # noqa: E501

        :return: The generation_id of this RelationModel.  # noqa: E501
        :rtype: QualifiedName
        """
        return self._generation_id

    @generation_id.setter
    def generation_id(self, generation_id):
        """Sets the generation_id of this RelationModel.

        It is an error to include a generation id in a relation other than DERIVATION  # noqa: E501

        :param generation_id: The generation_id of this RelationModel.  # noqa: E501
        :type: QualifiedName
        """

        self._generation_id = generation_id

    @property
    def usage_id(self):
        """Gets the usage_id of this RelationModel.  # noqa: E501

        It is an error to include a usage id in a relation other than DERIVATION  # noqa: E501

        :return: The usage_id of this RelationModel.  # noqa: E501
        :rtype: QualifiedName
        """
        return self._usage_id

    @usage_id.setter
    def usage_id(self, usage_id):
        """Sets the usage_id of this RelationModel.

        It is an error to include a usage id in a relation other than DERIVATION  # noqa: E501

        :param usage_id: The usage_id of this RelationModel.  # noqa: E501
        :type: QualifiedName
        """

        self._usage_id = usage_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RelationModel, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RelationModel):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

# flake8: noqa

"""
    Habitat repository and authorization API

    Habitat API  # noqa: E501

    OpenAPI spec version: V1.1.1
    Contact: zives@seas.upenn.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

# import apis into sdk package
from pennprov.api.authentication_api import AuthenticationApi
from pennprov.api.permission_api import PermissionApi
from pennprov.api.prov_dm_api import ProvDmApi
from pennprov.api.provenance_api import ProvenanceApi

# import ApiClient
from pennprov.api_client import ApiClient
from pennprov.configuration import Configuration
# import models into sdk package
from pennprov.models.attribute import Attribute
from pennprov.models.field_model import FieldModel
from pennprov.models.group_details import GroupDetails
from pennprov.models.id_model import IDModel
from pennprov.models.link_info import LinkInfo
from pennprov.models.link_instance import LinkInstance
from pennprov.models.node_info import NodeInfo
from pennprov.models.node_instance import NodeInstance
from pennprov.models.node_model import NodeModel
from pennprov.models.org_details import OrgDetails
from pennprov.models.prov_edge_model import ProvEdgeModel
from pennprov.models.prov_edge_set_model import ProvEdgeSetModel
from pennprov.models.prov_node_map_model import ProvNodeMapModel
from pennprov.models.prov_specifier_model import ProvSpecifierModel
from pennprov.models.prov_token_set_model import ProvTokenSetModel
from pennprov.models.qualified_name import QualifiedName
from pennprov.models.rank_instance import RankInstance
from pennprov.models.relation_model import RelationModel
from pennprov.models.response_error import ResponseError
from pennprov.models.store_link_model import StoreLinkModel
from pennprov.models.store_node_model import StoreNodeModel
from pennprov.models.subgraph_instance import SubgraphInstance
from pennprov.models.subgraph_template import SubgraphTemplate
from pennprov.models.tuple_with_schema_model import TupleWithSchemaModel
from pennprov.models.user_credentials import UserCredentials
from pennprov.models.user_info import UserInfo
from pennprov.models.web_token import WebToken
from pennprov.models.boolean_attribute import BooleanAttribute
from pennprov.models.boolean_field_model import BooleanFieldModel
from pennprov.models.double_attribute import DoubleAttribute
from pennprov.models.double_field_model import DoubleFieldModel
from pennprov.models.int_attribute import IntAttribute
from pennprov.models.integer_field_model import IntegerFieldModel
from pennprov.models.long_attribute import LongAttribute
from pennprov.models.long_field_model import LongFieldModel
from pennprov.models.multi_field_model import MultiFieldModel
from pennprov.models.prov_expression_model import ProvExpressionModel
from pennprov.models.prov_location_model import ProvLocationModel
from pennprov.models.prov_specifier_field_model import ProvSpecifierFieldModel
from pennprov.models.prov_token_field_model import ProvTokenFieldModel
from pennprov.models.prov_token_model import ProvTokenModel
from pennprov.models.qualified_name_attribute import QualifiedNameAttribute
from pennprov.models.qualified_name_field_model import QualifiedNameFieldModel
from pennprov.models.string_attribute import StringAttribute
from pennprov.models.string_field_model import StringFieldModel



# coding: utf-8

"""
    Habitat repository and authorization API

    Habitat API  # noqa: E501

    OpenAPI spec version: V1.1.1
    Contact: zives@seas.upenn.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from pennprov.models.prov_token_model import ProvTokenModel  # noqa: F401,E501
from pennprov.models.tuple_with_schema_model import TupleWithSchemaModel  # noqa: F401,E501


class StoreLinkModel(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        '_from': 'list[ProvTokenModel]',
        'to': 'ProvTokenModel',
        'label': 'str',
        'tuple_with_schema': 'TupleWithSchemaModel'
    }

    attribute_map = {
        '_from': 'from',
        'to': 'to',
        'label': 'label',
        'tuple_with_schema': 'tupleWithSchema'
    }

    def __init__(self, _from=None, to=None, label=None, tuple_with_schema=None):  # noqa: E501
        """StoreLinkModel - a model defined in Swagger"""  # noqa: E501

        self.__from = None
        self._to = None
        self._label = None
        self._tuple_with_schema = None
        self.discriminator = None

        self._from = _from
        self.to = to
        if label is not None:
            self.label = label
        if tuple_with_schema is not None:
            self.tuple_with_schema = tuple_with_schema

    @property
    def _from(self):
        """Gets the _from of this StoreLinkModel.  # noqa: E501


        :return: The _from of this StoreLinkModel.  # noqa: E501
        :rtype: list[ProvTokenModel]
        """
        return self.__from

    @_from.setter
    def _from(self, _from):
        """Sets the _from of this StoreLinkModel.


        :param _from: The _from of this StoreLinkModel.  # noqa: E501
        :type: list[ProvTokenModel]
        """
        if _from is None:
            raise ValueError("Invalid value for `_from`, must not be `None`")  # noqa: E501

        self.__from = _from

    @property
    def to(self):
        """Gets the to of this StoreLinkModel.  # noqa: E501


        :return: The to of this StoreLinkModel.  # noqa: E501
        :rtype: ProvTokenModel
        """
        return self._to

    @to.setter
    def to(self, to):
        """Sets the to of this StoreLinkModel.


        :param to: The to of this StoreLinkModel.  # noqa: E501
        :type: ProvTokenModel
        """
        if to is None:
            raise ValueError("Invalid value for `to`, must not be `None`")  # noqa: E501

        self._to = to

    @property
    def label(self):
        """Gets the label of this StoreLinkModel.  # noqa: E501


        :return: The label of this StoreLinkModel.  # noqa: E501
        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, label):
        """Sets the label of this StoreLinkModel.


        :param label: The label of this StoreLinkModel.  # noqa: E501
        :type: str
        """

        self._label = label

    @property
    def tuple_with_schema(self):
        """Gets the tuple_with_schema of this StoreLinkModel.  # noqa: E501


        :return: The tuple_with_schema of this StoreLinkModel.  # noqa: E501
        :rtype: TupleWithSchemaModel
        """
        return self._tuple_with_schema

    @tuple_with_schema.setter
    def tuple_with_schema(self, tuple_with_schema):
        """Sets the tuple_with_schema of this StoreLinkModel.


        :param tuple_with_schema: The tuple_with_schema of this StoreLinkModel.  # noqa: E501
        :type: TupleWithSchemaModel
        """

        self._tuple_with_schema = tuple_with_schema

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(StoreLinkModel, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StoreLinkModel):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from pennprov.api.authentication_api import AuthenticationApi
from pennprov.api.permission_api import PermissionApi
from pennprov.api.prov_dm_api import ProvDmApi
from pennprov.api.provenance_api import ProvenanceApi



# coding: utf-8

"""
    Habitat repository and authorization API

    Habitat API  # noqa: E501

    OpenAPI spec version: V1.1.1
    Contact: zives@seas.upenn.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from setuptools import setup, find_packages  # noqa: H301

NAME = "pennprov"
VERSION = "1.1.1.dev1"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = [
    "certifi>=2017.4.17",
    "python-dateutil>=2.1",
    "six>=1.10",
    "urllib3>=1.23"
]
    

setup(
    name=NAME,
    version=VERSION,
    description="Habitat repository and authorization API",
    author_email="zives@seas.upenn.edu",
    url="https://bitbucket.org/penndb/pennprov",
    keywords=["Swagger", "Habitat repository and authorization API"],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    long_description="""\
    Habitat API  # noqa: E501
    """
)



from __future__ import print_function
from pprint import pprint

import pennprov
from pennprov.rest import ApiException
from pennprov.metadata.stream_metadata import BasicSchema, BasicTuple


class MProvConnection:
    """
    MProvConnection is a high-level API to the PennProvenance framework, with
    a streaming emphasis (i.e., tuples are stored with positions or timestamps,
    and derivations are recorded each time an action is invoked).
    """

    namespace = 'http://mprov.md2k.org'
    graph_name = "mProv-graph"

    def __init__(self, user, password, host):
        # type: (str, str, str) -> None
        """
        Establish a connection to a PennProvenance server
        :param user: User ID
        :param password: Password for connection
        :param host: Host URL, or None to use localhost
        """
        self.configuration = pennprov.configuration.Configuration()
        if host:
            self.configuration.host = host
        else:
            self.configuration.host = "http://localhost:8088"

        api_client = pennprov.ApiClient(self.configuration)
        self.auth_api = pennprov.AuthenticationApi(api_client)
        self.prov_api = pennprov.ProvenanceApi(api_client)
        self.prov_dm_api = pennprov.ProvDmApi(api_client)
        self.username = user
        credentials = pennprov.UserCredentials(password)

        # One-time initialization of client with a JWT token for
        # authentication.
        web_token = self.auth_api.get_token_route(self.username, credentials)
        self.token = web_token.token

        self.configuration.api_key["api_key"] = self.token

        self.prov_api.create_or_reset_provenance_graph(self.get_graph())
        return

    def get_graph(self):
        """
        Within the storage system, get the name of the graph
        :return:
        """
        return self.graph_name


    def set_graph(self, name):
        """
        Set the name of the graph in the graph store
        :param name:
        :return:
        """
        self.graph_name = name

    def get_provenance_store(self):
        return self.prov_dm_api

    def get_low_level_api(self):
        return self.prov_api

    def get_auth_api(self):
        return self.auth_api

    def get_username(self):
        return self.username

    # Create a unique ID for an operator stream window
    def get_window_id(self, stream_operator, id):
        # type: (str, Any) -> str
        return stream_operator.join('_w.').join(str(id))

    # Create a unique ID for a stream operator result
    def get_result_id(self, stream, id):
        # type: (str, Any) -> str
        return stream.join('._r').join(id)

    # Create a unique ID for an entity
    def get_entity_id(self, stream, id):
        # type: (str, Any) -> str
        return stream + '._e' + str(id)

    # Create a unique ID for an activity (a stream operator call)
    def get_activity_id(self, operator, id):
        # type: (str, Any) -> str
        return operator + '._e' + str(id)

    def store_activity(self,
                       activity,
                       start,
                       end,
                       location):
        # type: (str, int, int, int) -> None
        """
        Create an entity node for an activity (a stream operator computation)

        :param activity: Name of the operation
        :param start: Start time
        :param end: End time
        :param location: Index position etc
        :return:
        """
        activity = pennprov.NodeModel(type='ACTIVITY',attributes=[], \
                                        location=location, start_time=start, end_time=end)
        self.prov_dm_api.store_node(resource=self.get_graph(),
                                    token=self.get_activity_id(activity, location), body=activity)
        return

    def store_stream_tuple(self, stream_name, stream_index, input_tuple):
        # type: (str, int, BasicTuple) -> pennprov.QualifiedName

        """
        Create an entity node for a stream tuple

        :param stream_name: The name of the stream itself
        :param stream_index: The index position (count) or timestamp (if unique)
        :param input_tuple: The actual stream value
        :return: token for the new node
        """

        # The "token" for the tuple will be the node ID
        token = pennprov.QualifiedName(self.namespace, self.get_entity_id(stream_name, stream_index - 1))

        # We also embed the token within the tuple, in case the programmer queries the database
        # and wants to see it
        input_tuple["_prov"] = self.get_entity_id(stream_name, stream_index - 1)

        # Now we'll create a tuple within the provenance node, to capture the data
        data = []
        for i, k in enumerate(input_tuple.schema.fields):
            qkey = pennprov.QualifiedName(self.namespace, k)
            data.append(pennprov.Attribute(name=qkey, value=input_tuple[i], type='STRING'))

        # Finally, we build an entity node within the graph, with the token and the
        # attributes
        entity_node = pennprov.NodeModel(type='ENTITY', attributes=data)
        self.prov_dm_api.store_node(resource=self.get_graph(),
                               token=token, body=entity_node)

        print('Storing node ' + str(token))

        return token

    def store_annotation(self,
                         stream_name,
                         stream_index,
                         annotation_name,
                         annotation_value):
        # type: (str, int, str, Any) -> pennprov.QualifiedName 
        """
        Create a node for an annotation to an entity / tuple

        :param stream_name: The name of the stream itself
        :param stream_index: The index position (count) or timestamp (if unique) of the
                stream element we are annotating
        :param annotation_name: The name of the annotation
        :param annotation_value: The value of the annotation
        :return:
        """


        # The "token" for the tuple will be the node ID
        token = pennprov.QualifiedName(self.namespace, self.get_entity_id(stream_name, stream_index - 1))

        ann_token = pennprov.QualifiedName(self.namespace,
                                           self.get_entity_id(stream_name + '.' + annotation_name, stream_index - 1))

        # The key/value pair will itself be an entity node
        attributes = [
            pennprov.Attribute(name=pennprov.QualifiedName(self.namespace, annotation_name), value=annotation_value,
                               type='STRING')]
        entity_node = pennprov.NodeModel(type='ENTITY', attributes=attributes)
        self.prov_dm_api.store_node(resource=self.get_graph(),
                               token=ann_token, body=entity_node)

        # Then we add a relationship edge (of type ANNOTATED)
        annotates = pennprov.RelationModel(
            type='ANNOTATED', subject_id=token, object_id=ann_token, attributes=[])
        self.prov_dm_api.store_relation(resource=self.get_graph(), body=annotates, label='_annotated')

        return ann_token

    def store_window_and_inputs(self,
                                output_stream_name,
                                output_stream_index,
                                input_tokens_list
                                ):
        # type: (str, int, list) -> pennprov.QualifiedName
        """
        Store a mapping between an operator window, from
        which a stream is to be derived, and the input
        nodes
        
        :param output_stream_name: 
        :param output_stream_index: 
        :param input_tokens_list: 
        :return: 
        """

        # The "token" for the tuple will be the node ID
        window_token = pennprov.QualifiedName(self.namespace, self.get_window_id(output_stream_name, output_stream_index - 1))
        # Finally, we build an entity node within the graph, with the token and the
        # attributes
        entity_node = pennprov.NodeModel(type='COLLECTION', attributes=[])
        self.prov_dm_api.store_node(resource=self.get_graph(),
                               token=window_token, body=entity_node)

        for token in input_tokens_list:
            # Add a relationship edge (of type ANNOTATED)
            # from window to its inputs
            token_qname = pennprov.QualifiedName(self.namespace, token)
            annotates = pennprov.RelationModel(
                type='MEMBERSHIP', subject_id=token_qname, object_id=window_token, attributes=[])
            self.prov_dm_api.store_relation(resource=self.get_graph(), body=annotates, label='membership')

        return window_token
    
    def store_windowed_result(self,
                                output_stream_name,
                                output_stream_index,
                                output_tuple,
                                input_tokens_list,
                                activity,
                                start,
                                end
                                ):
        # type: (str, int, BasicTuple, list, str, int, int) -> pennprov.QualifiedName
        """
        When we have a windowed computation, this creates a complex derivation subgraph
        in one operation.

        :param output_stream_name: The name of the stream our operator produces
        :param output_stream_index: The position of the outgoing tuple in the stream
        :param output_tuple: The tuple itself
        :param input_tokens_list: IDs of the inputs to the computation
        :param activity: The computation name
        :param start: Start time
        :param end: End time
        :return:
        """
        result_token = self.store_stream_tuple(output_stream_name, output_stream_index, output_tuple)
        window_token = self.store_window_and_inputs(output_stream_name, output_stream_index, input_tokens_list)

        activity_token = self.store_activity(activity, start, end, output_stream_index)

        derives = pennprov.RelationModel(
            type='DERIVATION', subject_id=result_token, object_id=window_token, attributes=[])
        self.prov_dm_api.store_relation(resource=self.get_graph(), body=derives, label='derivation')

        uses = pennprov.RelationModel(
            type='USAGE', subject_id=activity_token, object_id=window_token, attributes=[])
        self.prov_dm_api.store_relation(resource=self.get_graph(), body=uses, label='usage')

        generates = pennprov.RelationModel(
            type='GENERATION', subject_id=activity_token, object_id=result_token, attributes=[])
        self.prov_dm_api.store_relation(resource=self.get_graph(), body=uses, label='generation')

        return window_token



class BasicSchema:
    count = 0
    fields = []
    types = []

    def __init__(self, *args):
        if len(args) == 1:
            self.init1(args[0])
        elif len(args) == 2:
            self.init2(args[0], args[1])
        elif len(args) == 3:
            self.init3(args[0], args[1], args[2])
        else:
            raise TypeError('Illegal inputs')

    def init1(self, name):
        # type: (str) -> None
        self.name = name

    def init2(self, name, fields_types):
        # type: (str, dict) -> None
        """ Initialize with a name and a dictionary of attributes to types """
        self.name = name
        self.fields = []
        self.values = []
        for k in fields_types.keys():
            self.fields.append(k)
            self.types.append(fields_types[k])

    def init3(self, name, fields, types):
        # type: (str, list, list) -> None
        """ Initialize with a name and a list of field-type pairs """
        self.name = name
        self.fields = fields
        self.values = types

    def add_field(self, k, v):
        # type: (str, Any) -> None
        self.fields.append(k)
        self.types.append(v)

    def get_field(self, k):
        # type: (str) -> Any
        i = self.fields.index(k)
        if i:
            return self.types[i]
        else:
            return None

    def get_name(self):
        return self.name

    def create_tuple(self, *args):
        if len(args) == 0:
            return self.create_tuple_blank()
        elif isinstance(args[0], list):
            return self.create_tuple_list(args[0])
        else:
            return self.create_tuple_dict(args[0])

    def create_tuple_blank(self):
        return BasicTuple(self)

    def create_tuple_dict(self, content):
        # type: (dict) -> BasicTuple
        return BasicTuple(self, dict)

    def create_tuple_list(self, content):
        # type (list) -> BasicTuple
        dict2 = {}
        for i, k in enumerate(self.fields):
            dict2[k] = content[i]

        return BasicTuple(self, dict2)

    def __str__(self):
        ret = self.name + '('
        for i, f in enumerate(self.fields):
            if i > 0:
                ret = ret + ','
            ret = ret + f + ':' + self.types[i]

        return ret + ")"

class BasicTuple:
    schema = None
    data = {}

    def __init__(self, *args):
        if len(args) == 1:
            self.init1(args[0])
        elif len(args) == 2:
            self.init2(args[0], args[1])
        else:
            raise TypeError('Illegal inputs')

    def init1(self, schema):
        # type: (BasicSchema) -> None
        self.schema = schema
        for i, name in enumerate(schema.fields):
            self.data[name] = None

    def init2(self, schema, values):
        # type: (BasicSchema, dict) -> None
        self.schema = schema
        self.data = values

    def __str__(self):
        ret = self.schema.get_name() + '('
        for i, f in enumerate(self.schema.fields):
            if i > 0:
                ret = ret + ','
            if f is None:
                raise TypeError("Can't have a null key")
            elif self.data is None or self.data[f] is None:
                ret = ret + f
            else:
                ret = ret + f + ':' + str(self.data[f])

        return ret + ")"

    def __getitem__(self, item):
        if isinstance(item, int):
            return self.data[self.schema.fields[item]]
        else:
            return self.data[item]

    def __setitem__(self, key, value):
        if key in self.schema.fields:
            self.data[key] = value
        elif isinstance(key, int):
            self.data[self.schema.fields[key]] = value
